// Hey Emacs! This is -*- c++ -*- ! :-)
// DisjPartBdd.cc	-- Hans-Peter Stoerr Sun Feb  7 1999

#include <iostream.h>
#include <deque>
#include <algo.h>
#include "DisjPartBdd.h"
#include "misc/mystl.h"

Option  splitsize("split","Splitting size for transition relation.",8192);

class DisjPartBddConstructor
// abstract base class for strategy objects used in construction of DisjPartBdd
{
public:
  DisjPartBddConstructor(DisjPartBdd& theSink) : mSink(theSink), mSep("TransRel: [",",") , mCount(0),
    mBddGesSize(0) {}
  virtual ~DisjPartBddConstructor() {}

 // add new Bdd:
  DisjPartBddConstructor& operator<<(pair<Bdd,BddVector> bddInfo)
  { _AddBdd(bddInfo); mCount++; return *this; }
  // call when addition finished:
  void  Finish(Bdd reduceFilter) {
    _Finish(reduceFilter); *gpLog << "] (" << mBddGesSize << "|" << Size(mSink.mBddVector)
                                     << ") from " << mCount << " Bdds, "
                                     << mgr.NPrimedVars(0).size() << " vars/stateset.\n"
                                     << flush; }
  
protected:
  virtual void  _Finish(Bdd reduceFilter)=0; // this does the real work for Finish().
  virtual void  _AddBdd(const pair<Bdd,BddVector>)=0; // add a new Bdd
  void  PushBdd(Bdd b) { int s=b.Size(); *gpLog << mSep() << s << flush;
                         mBddGesSize+=s; mSink.mBddVector.push_back(b); }
private:
  DisjPartBdd&  mSink;
  FirsttimeNexttime<string> mSep;
  int   mCount,mBddGesSize;
};

// ================================================================

class DisjPartBddConstructorQueueing : public DisjPartBddConstructor
{
public:
  DisjPartBddConstructorQueueing(DisjPartBdd& theSink) : DisjPartBddConstructor(theSink) {}
  
protected:
  vector<pair<Bdd,BddVector> > mInput;
  deque<Bdd>    mQueue;

  virtual void  _Finish(Bdd reduceFilter);
  virtual void  _AddBdd(const pair<Bdd,BddVector>);
};

Option  actionsort("actionsort","Sort actions by support set",0);

// this is needed for BddBddMap.
struct BddCompareIndexInverse : public binary_function<Bdd, Bdd, bool> {
    bool operator()(const Bdd& x, const Bdd& y) const { return x.IfIndex()>y.IfIndex(); }
};

void DisjPartBddConstructorQueueing::_AddBdd(const pair<Bdd,BddVector> bddInfo)
{
  if(!actionsort)
    mQueue.push_back(bddInfo.first);
  else {
    mInput.push_back(bddInfo);
    sort(mInput.back().second.begin(), mInput.back().second.end(), BddCompareIndexInverse());
  }
}

static bool sortBddsByInfo(const pair<Bdd,BddVector>& a, const pair<Bdd,BddVector>& b) 
{
  return lexicographical_compare(a.second.begin(), a.second.end(),
                                 b.second.begin(), b.second.end(), BddCompareIndexInverse() );
}

void  DisjPartBddConstructorQueueing::_Finish(Bdd reduceFilter)
{
  if(actionsort) {
    sort(mInput.begin(), mInput.end(), sortBddsByInfo);
    transform(mInput.begin(), mInput.end(), back_inserter(mQueue), &first<Bdd,BddVector>);
  }
  while (mQueue.size()>=2) {
    Bdd a(mQueue.front()); mQueue.pop_front();
    Bdd b(mQueue.front()); mQueue.pop_front();
    long asiz=a.Size(), bsiz=b.Size();
    if (asiz>10*splitsize) { // pathological case if an actionbdd is too large
      PushBdd(a.Reduce(reduceFilter));
      mQueue.push_front(b);
    } else if (bsiz>10*splitsize) { // pathological case if an actionbdd is too large
      PushBdd(b.Reduce(reduceFilter));
      mQueue.push_front(b);
    } else {
      // set overflow to max((|a|+|b|)+100,splitsize), so we dont't generate
      // too large BDDs.
      long curlim=mgr.SetNodeLimit(mgr.GetNumNodes()+max(asiz+bsiz,long(splitsize)));
      Bdd ab=a+b;                 // we restrict the nodes this calculation may take
      bool overflow=mgr.WasOverflow();
      mgr.SetNodeLimit(curlim);   // set the old limit.
      long absiz=0;
      if (!overflow) absiz=ab.Size();
      if (!overflow && absiz<splitsize)
        mQueue.push_back(ab);
      else if (overflow || 4*absiz>5*(asiz+bsiz) ) {
        // we allow for a slight increase of absiz since we have less work at combining the parts.
        if(b.Size()>a.Size()) swap(a,b);
        PushBdd(a.Reduce(reduceFilter)); // push the larger of a,b
        mQueue.push_back(b);      // and keep the smaller.
      }
      else { // not overflow and absiz>=splitsize and 4*ab.Size()<=5*(a.Size()+b.Size())
        // PushBdd(ab.Reduce(reduceFilter));              // since we have reached splitsize
        mQueue.push_back(ab);     // since we had an reduction; may be we can reduce more.
      }
    }
  }
  if(0!=mQueue.size()) {
    PushBdd(mQueue.front().Reduce(reduceFilter)); mQueue.pop_front();
  }
  //   if (mBddVector.size()>1) {
  //     Bdd res=mBddVector.front();
  //     printit(res);
  //     BddVector::const_iterator it=mBddVector.begin()+1;
  //     for(; mBddVector.end()!=it; ++it)
  //       res=res.Reduce(-*it);
  //     printit(res);
  //   }
  assert(0==mQueue.size());
}

// ================================================================

DisjPartBdd::DisjPartBdd()
{
  mConstruct=new DisjPartBddConstructorQueueing(*this);
}

DisjPartBdd::~DisjPartBdd()
{
  if(NULL!=mConstruct) delete mConstruct;
}

DisjPartBdd& DisjPartBdd::operator +=(pair<Bdd,BddVector> bddInfo) // assigns a new part.
{
  assert(mConstruct!=NULL);
  *mConstruct << bddInfo;
  return *this;
}

void  DisjPartBdd::Finish(Bdd reduceFilter) // tells the DisjPartBdd that it now contains all parts.
{
  assert(mConstruct!=NULL);
  mConstruct->Finish(reduceFilter);
  delete mConstruct;
  mConstruct=NULL;
}

void  DisjPartBdd::Reduce(Bdd redBdd, const BddAssoc& redAss)
  // reduce Bdds through mult with redBdd and existential quantif. with redAss.
{
  assert(NULL==mConstruct);
  BddVector::iterator it=mBddVector.begin(), end=mBddVector.end();
  for (;end!=it;++it) *it=(*it*redBdd).Exists(redAss);
}

Option multiway("multiway","Use MultiwayOr");

Bdd     DisjPartBdd::operator*(Bdd in) const
{
  assert(NULL==mConstruct);
  if(multiway==0){ // use simple queueing algorithm
    deque<Bdd> queue;
    transform(mBddVector.begin(),mBddVector.end(),
              back_inserter(queue),bindthis(&Bdd::operator*,&in));
    while (queue.size()>1) {
      Bdd a(queue.front()); queue.pop_front();
      Bdd b(queue.front()); queue.pop_front();
      queue.push_back(a+b);
    }
    assert(1==queue.size());
    return queue.front();
  } else if (multiway==2) { // use sizesorted algorithm
    multimap<int,Bdd>   memory;
    BddVector::const_iterator it=mBddVector.begin(),end=mBddVector.end();
    for (;end!=it; ++it) {
      memory.insert(make_pair(it->Size(),*it));
    }
    while (1<memory.size()) {
      multimap<int,Bdd>::iterator it;
      it=memory.begin();
      Bdd first=it->second; memory.erase(it);
      it=memory.begin();
      Bdd second=it->second; memory.erase(it);
      Bdd both=first+second;
      memory.insert(make_pair(both.Size(),both));
    }
    return memory.begin()->second;
  } else {
    BddVector   resV;
    transform(mBddVector.begin(), mBddVector.end(),
              back_inserter(resV), bindthis(&Bdd::operator*,&in));
    return MultiwayOr(resV);
  }
}

Bdd   DisjPartBdd::RelProd(const BddAssoc& ass, Bdd in, double maxtime) const
{
  assert(NULL==mConstruct);
  double endtime=CpuTime()+maxtime;
  if(!multiway) {
    deque<Bdd> queue;
    BddVector::const_iterator it;
    for(it=mBddVector.begin(); it!=mBddVector.end(); it++) {
      queue.push_back(it->RelProd(ass,in));
      if (CpuTime()>endtime) {
        mgr.SetOverflowFlag();
        return Bdd();
      }
    }
    while (queue.size()>1) {
      Bdd a(queue.front()); queue.pop_front();
      Bdd b(queue.front()); queue.pop_front();
      Bdd ab=a+b;
      if (mgr.WasOverflow()) {
        mgr.SetOverflowFlag();
        return Bdd();
      }
      queue.push_back(ab);
      if (CpuTime()>endtime) {
        mgr.SetOverflowFlag();
        return Bdd();
      }
    }
    assert(1==queue.size());
    return queue.front();
  } else {
    BddVector   resV;
    BddVector::const_iterator it;
    for(it=mBddVector.begin(); it!=mBddVector.end(); it++) {
      resV.push_back(it->RelProd(ass,in));
      if (CpuTime()>endtime) {
        mgr.SetOverflowFlag();
        return Bdd();
      }
    }
    return MultiwayOr(resV);
  }
}

Bdd   DisjPartBdd::Exists(const BddAssoc& ass) const
{
  assert(NULL==mConstruct);
  if(!multiway) {
    deque<Bdd> queue;
    BddVector::const_iterator it;
    for(it=mBddVector.begin(); it!=mBddVector.end(); it++)
      queue.push_back(it->Exists(ass));
    while (queue.size()>1) {
      Bdd a(queue.front()); queue.pop_front();
      Bdd b(queue.front()); queue.pop_front();
      queue.push_back(a+b);
    }
    assert(1==queue.size());
    return queue.front();
  } else {
    BddVector   resV;
    BddVector::const_iterator it;
    for(it=mBddVector.begin(); it!=mBddVector.end(); it++)
      resV.push_back(it->Exists(ass));
    return MultiwayOr(resV);
  }
}

bool   DisjPartBdd::Intersects(Bdd p) const
  // returns true if (*this)*p != mgr.Zero()
{
  assert(NULL==mConstruct);
  BddVector::const_iterator it;
  for(it=mBddVector.begin(); it!=mBddVector.end(); it++)
    if (mgr.Zero()!= *it*p) return TRUE;
  return FALSE;
}

void    DisjPartBdd::Printit(ostream& s) const
{
  assert(NULL==mConstruct);
//   s << "Disjunctive Partitioned BDD with " << mBddVector.size() << " parts:\n";
//   for(BddVector::const_iterator it=mBddVector.begin(); mBddVector.end()!=it; ++it)
//     it->PrintInfo();
//   s << ".\n";
  FirsttimeNexttime<string> sep("",",");
  s << "DisjPartBdd with [";
  for(BddVector::const_iterator it=mBddVector.begin(); mBddVector.end()!=it; ++it)
    s << sep() << it->Size();
  s << "] nodes, " << mgr.NPrimedVars(0).size() << " vars/stateset.\n";
}
