/* FluentIterator.h -*- c++ -*- Hans-Peter Stoerr Thu Feb  4 1999 */
// iterates over all fluents of a domain
// this is only used once by Domain::MakeBddVarOrder,
// then the list of all fluents is available in Domain::mFluents.

#ifndef _FLUENTITERATOR_H
#define _FLUENTITERATOR_H

#include "misc/mystl.h"
#include <iterator.h>
using namespace std;

typedef vector<string>  Atomtype;
typedef map<string, Atomtype, less<string> > StringAtomtypeMap;
typedef vector<string> StringVector;

class Domain;

class FluentIterator : public Object, public input_iterator<string,void>
{
public:
  FluentIterator();             // yields end iterator
  FluentIterator(Domain&);      // yields begin iterator
  // FluentIterator(const FluentIterator&); // use compiler generated
  // FluentIterator& operator=(const FluentIterator&); // use compiler generated
  virtual       ~FluentIterator();

  bool  operator==(const FluentIterator&) const;
  bool  operator!=(const FluentIterator&) const;
  
  string        operator *();
  auto_ptr<string> operator ->() { return auto_ptr<string>(new string(**this)); }
  FluentIterator& operator++();
  // FluentIterator operator++(int) { FluentIterator res(*this); ++(*this); return res; }

  bool          IsFirstOfType() { return mIsFirst; }
  const string&         Fname() { return mFname; }
  const vector<string>& Fargs() { return mArgs; }
  const Atomtype&       Ftype() { return mFtypeIter->second; } // type of the fluent
  
protected:
  string        mFname;         // fluent name
  vector<string> mArgs;          // arguments of the fluent.
  bool          mIsFirst;
  StringAtomtypeMap::const_iterator     mFtypeIter,mFtypeEnd; // iterators for fluent type map
  // iterators for the objects of each argument:
  vector<StringVector::const_iterator> mBeginIters,mIters,mEndIters;
  Domain*       mpDom;           // the domain we are working on.

  bool          NextFluent(bool firsttime=FALSE); // goes to the next fluentname. returns true if there is one.
};

#endif

