/* LiteralQueue.h -*- c++ -*- unknown Sun Feb 27 2000 */

#ifndef _LITERALQUEUE_H
#define _LITERALQUEUE_H

#include <string>
#include <map>
#include <vector>
#include <list>
#include "Printable.h"

// it would be more efficient to use ints here, but at the moment we
// leave this at string for easier debugging.
typedef string  FNameT;         // type for Literalname
typedef string  ArgT;           // type for Argumentname
inline  bool    IsVariable(const ArgT& a) { return '?'==a[0]; } // Is Arg a Variable?
const   ArgT    AnonVariable="?";  // anonymous variablename

class CLiteral : public Printable
{ // this will probably change.
public:
  FNameT        mFName;
  vector<ArgT>  mArgs;

  CLiteral(){};
  CLiteral(FNameT n,ArgT a1,ArgT a2) : mFName(n) {mArgs.push_back(a1); mArgs.push_back(a2);}

protected:
  virtual void Printit(ostream &s) const
  {s << mFName << "(" << mArgs << ")";}
};

class LQCallback 
{
public:
  virtual void  LiteralAvailable(const CLiteral&)=0;
protected:                      // these are not available:
  LQCallback(){}; LQCallback(LQCallback&); virtual ~LQCallback(){};
  LQCallback& operator=(LQCallback&);
};

class LiteralQueue
// remembers all LQCallbacks for one Literaltype
{
public:
  LiteralQueue(FNameT fun,unsigned arity); ~LiteralQueue();
  FNameT        mFName;
  unsigned      mArity;

  void  InsertCallback(const CLiteral&,LQCallback*);
  void  LiteralAvailable(const CLiteral&,bool removeCallbacks=false);
  
private:
  class LQArg // choice structure for one argument of literal
  {
  public:
    LQArg(); ~LQArg();
    map<ArgT,LQArg>     mInstantiated;
    vector<LQCallback*>   mClients;
  };
  LQArg top;

  void LiteralAvailableRun(const CLiteral& l, bool removeCbs, unsigned narg, LQArg& mem);
};

#endif

