/* Printable.h	-*- c++ -*-  Hans-Peter Stoerr Mon Jan 11 1999 */

#ifndef _PRINTABLE_H
#define _PRINTABLE_H

#include <assert.h>
#include <ostream>
#include <string>
#include <vector>
#include <list>
// #include <slist>
#include <map>
#include <set>

using namespace std;

#include "general.h"

class Printable : public Object
{
  friend ostream&   operator<<(ostream& s, const Printable& x);
public:
  virtual ~Printable() {}
protected:
  virtual void Printit(ostream &s) const=0;
private:
  void    p();                  // printing function to be called from gdb.
};

inline ostream&   operator<<(ostream& s, const Printable& x)
{
  x.Printit(s);
  return s;
}

template<class T> ostream& operator<<(ostream& s, const list<T>& l)
{
  typename list<T>::const_iterator it=l.begin();
  if(it!=l.end()){
    s << *it;
    it++;
  }
  while(it!=l.end()) {
    s << ", " << *it;
    it++;
  }
  return s;
}


template<class T> ostream& operator <<
(ostream &s, const vector<T>& l)
{
  typename vector<T>::const_iterator it=l.begin();
  s << "[";
  if(it!=l.end()){
    s << *it;
    it++;
  }
  while(it!=l.end()) {
    s << ", " << *it;
    it++;
  }
  s << "]";
  return s;
}

template<class T,class C> ostream& operator <<
(ostream &s, const set<T,C>& l)
{
  typename set<T,C>::const_iterator it=l.begin();
  s << "{";
  if(it!=l.end()){
    s << *it;
    it++;
  }
  while(it!=l.end()) {
    s << ", " << *it;
    it++;
  }
  s << "}";
  return s;
}

template<class T,class T2,class C> ostream& operator <<
(ostream &s, const map<T,T2,C>& l)
{
  typename map<T,T2,C>::const_iterator it=l.begin();
  if(it!=l.end()){
    s << (*it).first << " => " << (*it).second;
    it++;
  }
  while(it!=l.end()) {
    s << "\n\t" << (*it).first << " => " << (*it).second;
    it++;
  }
  return s;
}

template <class T1, class T2> ostream& operator <<
(ostream &s, const pair<T1,T2>& p)
{
  s << "(" << p.first << " - " << p.second << ")" ;
  return s;
}

#define printit(y) do{ cout << __FILE__ << ":" << __LINE__ << ":" \
    << #y << "=\t" << (y) << ".\n"; } while(0)

#endif
