/* Term.h -*- c++ -*- Hans-Peter Stoerr Mon Jan 11 1999 */

#ifndef _TERM_H
#define _TERM_H

#include <assert.h>
#include <string>
#include <vector>
#include <list>
#include <set>
#include <map>
#include "Printable.h"
using namespace std;

typedef list<string> StringList;
typedef set<string,less<string> > StringSet;
typedef map<string, string, less<string> > StringStringMap;

class TermRep;

class Term: public Printable
{
  public:
  enum TermType {Not,And,Or,Imply,When,Special,Forall,Exists,Equal,Atom,Null};
  
  Term();
  Term(const Term&);
  virtual ~Term();
  Term& operator=(const Term&);

  // This one should really be protected:, but I cannot make MakeTerm friend
  // w/o problems because I cannot declare TermSet etc. yet. :-(
  Term(TermRep*);               // increments reference counter of rep.

  int Cmp(const Term& t2) const;
  friend bool operator<(const Term&, const Term&);
  TermRep* operator->() { return rep; }
  const TermRep* operator->() const { return rep; }
  TermRep& operator*() { return *rep; }
  const TermRep& operator*() const { return *rep; }

  TermType Type() const;
  bool  IsEmpty() const;
  string Describe() const;      // returns string representation
protected:
  virtual void Printit(ostream& s) const;
  TermRep       *rep;
  bool operator==(const Term&) const; // should be public and implemented, but...
};

class TermRep: public Printable
// abstract base class for representationobject of Terms
{
  friend class Term;
public:
  string name;
  const Term::TermType type;
protected:
  int refs;                     // reference count

  TermRep(Term::TermType thetype) : type(thetype) { refs=0; }
  virtual ~TermRep() {};
  
  virtual int Cmp(const TermRep&) const=0; // compare; assumes same type.
  Term  Clone() { return Term(this); } // returns new copy of Term based on *this.
private:
  TermRep();                    // unimplemented, can't be done.
  TermRep(const TermRep&);       // unimplemented, can't be done.
  TermRep& operator=(const TermRep&); // unimplemented, can't be done.
};

inline Term::Term() : rep(NULL) {}

inline Term::Term(const Term& t) : Printable(t), rep(t.rep) {if(rep) rep->refs++;}

inline Term::~Term()
{
  if(rep) {
    rep->refs--; assert(rep->refs>=0);
    if(rep->refs==0) delete rep;
  }
}

inline Term& Term::operator=(const Term& t) 
{
  if(t.rep) t.rep->refs++;
  if(rep){
    rep->refs--; assert(rep->refs>=0);
    if(rep->refs==0) delete rep;
  }
  rep=t.rep;
  return *this;
}

inline Term::TermType Term::Type() const
{ if (rep) return rep->type; else return Term::Null; }

inline bool Term::IsEmpty() const
{ return NULL==rep; }

typedef set< Term, less<Term> > TermSet;
typedef list< Term > TermList;
typedef vector< Term > TermVector;

Term MakeTerm(Term::TermType, TermSet args);
Term MakeTerm(Term::TermType, Term arg);
Term MakeTerm(Term::TermType, string fname, Term arg); // for Special
Term MakeTerm(Term::TermType, Term arg1, Term arg2);
Term MakeTerm(string predicate, StringList args);
Term MakeTerm(Term::TermType, string arg1, string arg2);
Term MakeTerm(Term::TermType type, StringStringMap vars, Term arg);

#endif
