// Hey Emacs! This is -*- c++ -*- ! :-)
// bddplan.cc	-- Hans-Peter Stoerr Thu Jan  7 1999

#include <sys/time.h>
#include <sys/resource.h>
#include <sys/times.h>
#include <limits.h>
#include <signal.h>
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>

#include <iostream>
#include <fstream>
#include "misc/misc.h"
#include <functional>

using namespace std;

#include "general.h"
#include "pddldomain.h"
#include "Domain.h"
#include "BddTransPlan.h"
#include "FluentIterator.h"

#ifdef sun
extern "C" 
{
  int getrusage(int who, struct rusage *rusage);
}
#endif

#if !defined(linux)
extern "C" { long random(); }
#endif

extern int gPrintWarnings;  /* cal.h. reset this, if we should not print warnings.  */

const   char*   gProgName;
ostream*        gpLog=&cout;
ostream*        gpRes=&cout;
static string   sResultString; // contains result that will be written to gpRes.

int     gTimeLimit;

void    SetTimeLimit(int timeseconds)
{
  struct rlimit rl;
  getrlimit(RLIMIT_CPU,&rl);
  rl.rlim_cur=timeseconds*1000;
  rl.rlim_max=timeseconds*1000;
  gTimeLimit=timeseconds;
  if (0!=setrlimit(RLIMIT_CPU,&rl))
    perror("setrlimit");
  alarm(timeseconds+1); // :KLUDGE: setrlimit seems not to work sometimes.
}

void    SetMemoryLimit(int memmegs)
{
  struct rlimit rl;
  #if linux
  #define MEMLIMIT RLIMIT_DATA
  #else
  #define MEMLIMIT RLIMIT_VMEM
  #endif
  getrlimit(MEMLIMIT,&rl);
  rl.rlim_cur=memmegs*1024*1024; // this seems not completely right.
  rl.rlim_max=rl.rlim_cur;
  if (0!=setrlimit(MEMLIMIT,&rl))
    perror("setrlimit");
  getrlimit(RLIMIT_CORE,&rl);
  rl.rlim_cur=0;
  rl.rlim_max=0;
  if (0!=setrlimit(RLIMIT_CORE,&rl))
    perror("setrlimit");
}

double  CpuTime()               // return used CPU time in seconds.
{
  struct tms buf; times(&buf);
  return double(buf.tms_utime)/sysconf(_SC_CLK_TCK);
  // return double(buf.tms_utime+buf.tms_stime)/CLK_TCK;
}

string  MakeString(int i)
{
  char  s[1000];
  sprintf(s,"v%02d",i);
  return s;
}

static bool rndBool()
{
  long res=random();
  return ((res*(res/1<<16))>>16)%2;
}

void    CrashRun() 
{ // produce really big BDD fast.
  const int maxvar=100;
  int varnum,i;
  BddVector     vs;
  cout << "Warning: Crashmode starting (i.e. produce as large BDD as possible).\n" << flush;
  for(varnum=0;varnum<maxvar;varnum++) vs.push_back(Bdd(MakeString(varnum)));
  Bdd riesiggross=mgr.Zero();
  while (!mgr.WasOverflow()) {
      Bdd newxor=mgr.One();
      for (i=0;i<varnum;++i) if (rndBool()) newxor^=vs[i];
      riesiggross+=newxor;
  }
  cout << "Overflow.\n" << flush;
  sleep(20);
}

void timeoutexit(int) { *gpLog << "Timeout.\n" << flush; cout << flush; cerr << flush; exit(1); }

void alarmhandler(int arg)
{
  signal(SIGALRM,alarmhandler);
  if(CpuTime()>gTimeLimit) timeoutexit(arg);
  alarm( int(gTimeLimit-CpuTime())+1 );
}

// struct rusage gUsage;
static struct startupcleanup
{
  startupcleanup() { cout.sync_with_stdio(); signal(SIGXCPU,timeoutexit); signal(SIGALRM,alarmhandler); }
  ~startupcleanup()
  {
    *gpLog // << "Mem: " << gUsage.ru_ixrss+gUsage.ru_idrss+gUsage.ru_isrss
           << "CPUtime: " << long(CpuTime()*1000) << "ms.#\n" << flush;
    if (gpRes!=NULL) *gpRes << sResultString << "\n" << flush;
    if (&cout!=gpLog) { dynamic_cast<ofstream*>(gpLog)->close(); delete gpLog; }
    if (gpRes!=NULL && &cout!=gpRes) {
      dynamic_cast<ofstream*>(gpRes)->close(); delete gpRes; }
  }
} do_startupcleanup;

void    usage(bool called)
{
  ostream *out;
  if (called) out=&cout; else out=&cerr;
  *out << "Usage: " << gProgName
       << " [-v] [-w] [-c] [-p Problemname] [-d Domainname] [-l Logfile]"
       << " [-r Resultfile] [-z Parseroutputfile] [-o Options]* [-n nodelimit]"
       << " [-m memorylimit(megs)] [-t timelimit(sec)]  [filename.pddl]*\n"
       << "-v: list problems, -w: list domains, -c: check only\n";
  DescribeOptions(called);
}

string  LispFormat(string action)
{
  string::size_type pos=0;
  pos=action.find_first_of("(,");
  while (action.npos!=pos) {
    action.replace(pos,1," ");
    pos=action.find_first_of("(,");
  }
  pos=action.find(" )");
  if (action.npos!=pos) action.erase(pos,1);
  return "("+action;
}

Option  timing("timing","Print Timings of some crucial operations.");
extern Option movetypes;

int main(const int argc, char*const* argv) // ,const char *const* envp
{
  ofstream out("/dev/null");  // send some debugging output there.
  // Some Variables we intend to get from options
  vector<string> filenames;
  string problemname="",domainname="";
  bool listproblems=FALSE,listdomains=FALSE,parseonly=FALSE;
  gProgName=argv[0];
  // we now parse options.
  char c;
  extern char *optarg;
  extern int optind;
  mgr.SetNodeLimit(10000000);
  while ( EOF!=(c=getopt(argc,argv,"p:d:o:l:r:n:t:m:z:GqvwcXh")) )
    switch(c) {
    case 'p':
      problemname=optarg;
      break;
    case 'd':
      domainname=optarg;
      break;
    case 'o':
      SetOptions(optarg);
      break;
    case 'l':                   // logfile
      gpLog=new ofstream(optarg,ios::out|ios::app);
      if(!*gpLog) {
        perror((gProgName+string(": logfile ")+optarg).c_str());
        return 2;
      }
      break;
    case 'r':                   // resultsfile
      gpRes=new ofstream(optarg,ios::out|ios::app);
      if(!*gpRes) {
        perror((gProgName+string(": resultfile ")+optarg).c_str());
        return 2;
      }
      break;
    case 'z':                   // output parsed pddl
      out.close(); out.open(optarg);
      break;
    case 'q':                   // suppress unnecessary warnings etc.
      gPrintWarnings=0;
      break;
    case 'v':
      listproblems=1;
      gpLog=new ofstream("/dev/null");
      break;
    case 'w':
      listdomains=1;
      gpLog=new ofstream("/dev/null");
      break;
    case 'n':                   // nodelimit
      mgr.SetNodeLimit(atoi(optarg));
      break;
    case 't':                   // timelimit
      SetTimeLimit(atoi(optarg));
      break;
    case 'm':
      SetMemoryLimit(atoi(optarg));
      break;
    case 'X':
      CrashRun(); return 0;
    case 'c':                   // check only
      parseonly=TRUE;
      break;
    case 'G':
      mgr.EnableGC(FALSE);
      break;
    case 'h':
    default:
      usage(c=='h');
      return c=='h';
    }
  for (; optind<argc; optind++)
    filenames.push_back(argv[optind]);
  if ( 0==filenames.size() ) filenames.push_back("a.pddl");
  if(&cout!=gpLog) *gpLog << "\n";
  if(!out) perror((string(gProgName)+": out").c_str());
  out << flush;
  PddlDatabase P;
  P.ParseFiles(filenames);
  if(listproblems && listdomains) {
    P.ListProblemsWithDomains();
    return 0;
  }
  if(listproblems) {
    P.ListProblems();
    return 0;
  }
  if(listdomains) {
    P.ListDomains();
    return 0;
  }
  out << P << flush;
  Domain D(P,domainname,problemname);
  PrintOptions();
  if(parseonly) return 0;
  D.Init();
  if (movetypes) { // if we have moved types we might have changed D.constants now
    out << "----------------------------------------------------------------\n";
    out << D.dom.constants;
  }
  out << "================================================================\n";
  out << D;
  out << "================================================================\n";
  out << flush;
  BddTransPlan Planner(D);
  cout << flush;
  cerr << flush;
  *gpLog << flush;
  Planner.Run();
  *gpLog << "Peak Nodecount: " << mgr.PeakNumNodes() << ", " << D.mFluents.size() << " Fluents, " << flush;
//   { // check variable ordering.
//     BddVector   op(mgr.NPrimedVars(0)), p(mgr.NPrimedVars(1));
//     sort(op.begin(),op.end(),Bdd::BddIndexCompare());
//     sort(p.begin(),p.end(),Bdd::BddIndexCompare());
//     for(int i=0;i<op.size();i++) {
//       // cout << op[i].VarName() << ": " << op[i].IfIndex() << ", " << p[i].IfIndex() << "\n";
//       assert(op[i].VarName()+"`"==p[i].VarName());
//       assert(op[i].IfIndex()+1==p[i].IfIndex());
//     }
//   }
  // write plan in AIPS competition-format to sResultString
  char  resstr[1000];
  sprintf(resstr,"%f,%d",CpuTime(),Planner.mPlan.size());
  sResultString=D.prob.name+","+resstr;
  StringVector::const_iterator it=Planner.mPlan.begin(), end=Planner.mPlan.end();
  for (;end!=it;++it) sResultString+=","+LispFormat(*it);
  // getrusage(RUSAGE_SELF,&gUsage); perror("getrusage");
  return 0;
}

