/* general.h -*- c++ -*- Hans-Peter Stoerr Tue Feb  2 1999 */
/* Some general stuff to be included everywhere. */

#ifndef _GENERAL_H
#define _GENERAL_H

#include "Options.h"

// typedef int bool;

const bool TRUE=1;
const bool FALSE=0;

extern ostream*   gpLog;
extern const char*      gProgName;        // programname used for error output

class Object
{
#if DEBUG
public:
  virtual ~Object(){}
#endif
#if CHECKDESTRUCTION
public:
  Object();
  Object& operator=(const Object&) {return *this;}
  Object(const Object&);
private:
  const  long  mNumber;
  static long Count;
#endif
};

#define CHECK(expression,errordescription) do { \
  if (!(expression)) { \
    *gpLog << __FILE__ << ":" << __LINE__ << ": check (" << #expression << ") failed: " \
           << errordescription << "\n" << flush; \
    abort(); \
  } \
} \
while (0)

// this aborts on call of an unimplemented functionality
#define UNIMPLEMENTED CHECK(FALSE,"Unimplemented feature.")
// this marks empty bodies
#define NOOP do { } while(0)
// this aborts on a branch that shouldn't happen (i.e. bug.)
#define IMPOSSIBLE CHECK(FALSE,"This shouldn't happen.")

extern double CpuTime();        // return used CPU time in seconds.

void    PrintMyTiming(double time,char* file,int line,char* command);

#define Timing(X) do {double strt=CpuTime(); {X;}; double end=CpuTime(); \
  PrintMyTiming(end-strt,__FILE__,__LINE__,#X); } while(0)           

#define GetTiming(T,X) do {double strt=CpuTime(); {X;}; double end=CpuTime(); \
  T=end-strt; } while(0)           

#endif
