/* Lazy.h -*- c++ -*- Hans-Peter Stoerr Fri May 28 1999 */
// lazy evaluation classes

#ifndef _LAZY_H
#define _LAZY_H

template <class T> class LazyValue 
{
public:
  LazyValue();
  LazyValue(T val);
  LazyValue(const LazyValue&);
  virtual ~LazyValue();
  LazyValue&    operator=(const LazyValue&);
  operator T();
protected:
  class LazyRep 
  {
  public:
    LazyRep();
    LazyRep(const LazyRep&);
    virtual ~LazyRep();
    virtual LazyRep& operator=(const LazyRep&)=0;

    virtual T calculate()=0;
    T   value() {if(!mEvaluated){mVal=calculate(); mEvaluated=1;} return mVal;}
  private:
    T           mVal;
    bool        mEvaluated;
    int         mRefs;
  };
  LazyRep       *mRep;
};

template <class T> class LazyRep0 : public LazyValue<T>::LazyRep 
{
};

template <class Arg1,class T> class LazyRep1 : public LazyValue<T>::LazyRep
{
public:
  LazyRep1();
  LazyRep1(const LazyRep1&);
  typename LazyValue<T>::LazyRep& operator =(const typename LazyValue<T>::LazyRep&);
private:
  LazyValue<Arg1> mA1;
};

template <class Arg,class T>
class LazyFun{
  T (*mFun)(Arg);
public:
  LazyFun(T (*fun)(Arg)) : mFun(fun) {};
  LazyValue<T> operator()(LazyValue<Arg> arg);
};

#endif
