; the towers of hanoi problem
;
; I think this is interesting because it requires exponential length plans,
; which is something only few planners can handle.
; plan length for n disks is 2^n-1 :serial as well as :parallel.
; Hans-Peter Stoerr <hans-peter@go.to>

(define (domain towers-of-hanoi)
  (:requirements :typing :adl)
  (:types disk peg)
  (:constants pega pegb pegc - peg)
  (:predicates (on ?d - disk ?p - peg)
               (larger ?d1 ?d2 - disk)) ; disk d1 is larger than d2
  (:action move
           :parameters (?d - disk ?p1 ?p2 - peg)
           :precondition (and (on ?d ?p1) (not (= ?p1 ?p2))
                              (not (exists (?d1 - disk)
                                           (and (on ?d1 ?p1) (larger ?d ?d1))))
                              (not (exists (?d1 - disk)
                                           (and (on ?d1 ?p2) (larger ?d ?d1)))))
           :effect (and (not (on ?d ?p1)) (on ?d ?p2))))

(define (problem hanoi-2)
  (:domain towers-of-hanoi)
  (:objects d1 d2 - disk)
  (:init (on d1 pega) (on d2 pega) 
         (larger d2 d1) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc))))

(define (problem hanoi-3)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc))))

(define (problem hanoi-4)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc))))

(define (problem hanoi-5)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc))))

(define (problem hanoi-6)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc))))

(define (problem hanoi-7)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc))))

(define (problem hanoi-8)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc))))

(define (problem hanoi-9)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc))))

(define (problem hanoi-10)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc))))

(define (problem hanoi-11)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc))))

(define (problem hanoi-12)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc))))

(define (problem hanoi-13)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) (on d13 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         (larger d13 d1) (larger d13 d2) (larger d13 d3) (larger d13 d4) (larger d13 d5) (larger d13 d6) (larger d13 d7) (larger d13 d8) (larger d13 d9) (larger d13 d10) (larger d13 d11) (larger d13 d12) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc) (on d13 pegc))))

(define (problem hanoi-14)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 d14 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) (on d13 pega) (on d14 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         (larger d13 d1) (larger d13 d2) (larger d13 d3) (larger d13 d4) (larger d13 d5) (larger d13 d6) (larger d13 d7) (larger d13 d8) (larger d13 d9) (larger d13 d10) (larger d13 d11) (larger d13 d12) 
         (larger d14 d1) (larger d14 d2) (larger d14 d3) (larger d14 d4) (larger d14 d5) (larger d14 d6) (larger d14 d7) (larger d14 d8) (larger d14 d9) (larger d14 d10) (larger d14 d11) (larger d14 d12) (larger d14 d13) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc) (on d13 pegc) (on d14 pegc))))

(define (problem hanoi-15)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 d14 d15 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) (on d13 pega) (on d14 pega) (on d15 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         (larger d13 d1) (larger d13 d2) (larger d13 d3) (larger d13 d4) (larger d13 d5) (larger d13 d6) (larger d13 d7) (larger d13 d8) (larger d13 d9) (larger d13 d10) (larger d13 d11) (larger d13 d12) 
         (larger d14 d1) (larger d14 d2) (larger d14 d3) (larger d14 d4) (larger d14 d5) (larger d14 d6) (larger d14 d7) (larger d14 d8) (larger d14 d9) (larger d14 d10) (larger d14 d11) (larger d14 d12) (larger d14 d13) 
         (larger d15 d1) (larger d15 d2) (larger d15 d3) (larger d15 d4) (larger d15 d5) (larger d15 d6) (larger d15 d7) (larger d15 d8) (larger d15 d9) (larger d15 d10) (larger d15 d11) (larger d15 d12) (larger d15 d13) (larger d15 d14) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc) (on d13 pegc) (on d14 pegc) (on d15 pegc))))

(define (problem hanoi-16)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 d14 d15 d16 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) (on d13 pega) (on d14 pega) (on d15 pega) (on d16 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         (larger d13 d1) (larger d13 d2) (larger d13 d3) (larger d13 d4) (larger d13 d5) (larger d13 d6) (larger d13 d7) (larger d13 d8) (larger d13 d9) (larger d13 d10) (larger d13 d11) (larger d13 d12) 
         (larger d14 d1) (larger d14 d2) (larger d14 d3) (larger d14 d4) (larger d14 d5) (larger d14 d6) (larger d14 d7) (larger d14 d8) (larger d14 d9) (larger d14 d10) (larger d14 d11) (larger d14 d12) (larger d14 d13) 
         (larger d15 d1) (larger d15 d2) (larger d15 d3) (larger d15 d4) (larger d15 d5) (larger d15 d6) (larger d15 d7) (larger d15 d8) (larger d15 d9) (larger d15 d10) (larger d15 d11) (larger d15 d12) (larger d15 d13) (larger d15 d14) 
         (larger d16 d1) (larger d16 d2) (larger d16 d3) (larger d16 d4) (larger d16 d5) (larger d16 d6) (larger d16 d7) (larger d16 d8) (larger d16 d9) (larger d16 d10) (larger d16 d11) (larger d16 d12) (larger d16 d13) (larger d16 d14) (larger d16 d15) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc) (on d13 pegc) (on d14 pegc) (on d15 pegc) (on d16 pegc))))

(define (problem hanoi-17)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 d14 d15 d16 d17 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) (on d13 pega) (on d14 pega) (on d15 pega) (on d16 pega) (on d17 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         (larger d13 d1) (larger d13 d2) (larger d13 d3) (larger d13 d4) (larger d13 d5) (larger d13 d6) (larger d13 d7) (larger d13 d8) (larger d13 d9) (larger d13 d10) (larger d13 d11) (larger d13 d12) 
         (larger d14 d1) (larger d14 d2) (larger d14 d3) (larger d14 d4) (larger d14 d5) (larger d14 d6) (larger d14 d7) (larger d14 d8) (larger d14 d9) (larger d14 d10) (larger d14 d11) (larger d14 d12) (larger d14 d13) 
         (larger d15 d1) (larger d15 d2) (larger d15 d3) (larger d15 d4) (larger d15 d5) (larger d15 d6) (larger d15 d7) (larger d15 d8) (larger d15 d9) (larger d15 d10) (larger d15 d11) (larger d15 d12) (larger d15 d13) (larger d15 d14) 
         (larger d16 d1) (larger d16 d2) (larger d16 d3) (larger d16 d4) (larger d16 d5) (larger d16 d6) (larger d16 d7) (larger d16 d8) (larger d16 d9) (larger d16 d10) (larger d16 d11) (larger d16 d12) (larger d16 d13) (larger d16 d14) (larger d16 d15) 
         (larger d17 d1) (larger d17 d2) (larger d17 d3) (larger d17 d4) (larger d17 d5) (larger d17 d6) (larger d17 d7) (larger d17 d8) (larger d17 d9) (larger d17 d10) (larger d17 d11) (larger d17 d12) (larger d17 d13) (larger d17 d14) (larger d17 d15) (larger d17 d16) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc) (on d13 pegc) (on d14 pegc) (on d15 pegc) (on d16 pegc) (on d17 pegc))))

(define (problem hanoi-18)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 d14 d15 d16 d17 d18 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) (on d13 pega) (on d14 pega) (on d15 pega) (on d16 pega) (on d17 pega) (on d18 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         (larger d13 d1) (larger d13 d2) (larger d13 d3) (larger d13 d4) (larger d13 d5) (larger d13 d6) (larger d13 d7) (larger d13 d8) (larger d13 d9) (larger d13 d10) (larger d13 d11) (larger d13 d12) 
         (larger d14 d1) (larger d14 d2) (larger d14 d3) (larger d14 d4) (larger d14 d5) (larger d14 d6) (larger d14 d7) (larger d14 d8) (larger d14 d9) (larger d14 d10) (larger d14 d11) (larger d14 d12) (larger d14 d13) 
         (larger d15 d1) (larger d15 d2) (larger d15 d3) (larger d15 d4) (larger d15 d5) (larger d15 d6) (larger d15 d7) (larger d15 d8) (larger d15 d9) (larger d15 d10) (larger d15 d11) (larger d15 d12) (larger d15 d13) (larger d15 d14) 
         (larger d16 d1) (larger d16 d2) (larger d16 d3) (larger d16 d4) (larger d16 d5) (larger d16 d6) (larger d16 d7) (larger d16 d8) (larger d16 d9) (larger d16 d10) (larger d16 d11) (larger d16 d12) (larger d16 d13) (larger d16 d14) (larger d16 d15) 
         (larger d17 d1) (larger d17 d2) (larger d17 d3) (larger d17 d4) (larger d17 d5) (larger d17 d6) (larger d17 d7) (larger d17 d8) (larger d17 d9) (larger d17 d10) (larger d17 d11) (larger d17 d12) (larger d17 d13) (larger d17 d14) (larger d17 d15) (larger d17 d16) 
         (larger d18 d1) (larger d18 d2) (larger d18 d3) (larger d18 d4) (larger d18 d5) (larger d18 d6) (larger d18 d7) (larger d18 d8) (larger d18 d9) (larger d18 d10) (larger d18 d11) (larger d18 d12) (larger d18 d13) (larger d18 d14) (larger d18 d15) (larger d18 d16) (larger d18 d17) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc) (on d13 pegc) (on d14 pegc) (on d15 pegc) (on d16 pegc) (on d17 pegc) (on d18 pegc))))

(define (problem hanoi-19)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 d14 d15 d16 d17 d18 d19 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) (on d13 pega) (on d14 pega) (on d15 pega) (on d16 pega) (on d17 pega) (on d18 pega) (on d19 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         (larger d13 d1) (larger d13 d2) (larger d13 d3) (larger d13 d4) (larger d13 d5) (larger d13 d6) (larger d13 d7) (larger d13 d8) (larger d13 d9) (larger d13 d10) (larger d13 d11) (larger d13 d12) 
         (larger d14 d1) (larger d14 d2) (larger d14 d3) (larger d14 d4) (larger d14 d5) (larger d14 d6) (larger d14 d7) (larger d14 d8) (larger d14 d9) (larger d14 d10) (larger d14 d11) (larger d14 d12) (larger d14 d13) 
         (larger d15 d1) (larger d15 d2) (larger d15 d3) (larger d15 d4) (larger d15 d5) (larger d15 d6) (larger d15 d7) (larger d15 d8) (larger d15 d9) (larger d15 d10) (larger d15 d11) (larger d15 d12) (larger d15 d13) (larger d15 d14) 
         (larger d16 d1) (larger d16 d2) (larger d16 d3) (larger d16 d4) (larger d16 d5) (larger d16 d6) (larger d16 d7) (larger d16 d8) (larger d16 d9) (larger d16 d10) (larger d16 d11) (larger d16 d12) (larger d16 d13) (larger d16 d14) (larger d16 d15) 
         (larger d17 d1) (larger d17 d2) (larger d17 d3) (larger d17 d4) (larger d17 d5) (larger d17 d6) (larger d17 d7) (larger d17 d8) (larger d17 d9) (larger d17 d10) (larger d17 d11) (larger d17 d12) (larger d17 d13) (larger d17 d14) (larger d17 d15) (larger d17 d16) 
         (larger d18 d1) (larger d18 d2) (larger d18 d3) (larger d18 d4) (larger d18 d5) (larger d18 d6) (larger d18 d7) (larger d18 d8) (larger d18 d9) (larger d18 d10) (larger d18 d11) (larger d18 d12) (larger d18 d13) (larger d18 d14) (larger d18 d15) (larger d18 d16) (larger d18 d17) 
         (larger d19 d1) (larger d19 d2) (larger d19 d3) (larger d19 d4) (larger d19 d5) (larger d19 d6) (larger d19 d7) (larger d19 d8) (larger d19 d9) (larger d19 d10) (larger d19 d11) (larger d19 d12) (larger d19 d13) (larger d19 d14) (larger d19 d15) (larger d19 d16) (larger d19 d17) (larger d19 d18) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc) (on d13 pegc) (on d14 pegc) (on d15 pegc) (on d16 pegc) (on d17 pegc) (on d18 pegc) (on d19 pegc))))

(define (problem hanoi-20)
  (:domain towers-of-hanoi)
  (:objects d1 d2 d3 d4 d5 d6 d7 d8 d9 d10 d11 d12 d13 d14 d15 d16 d17 d18 d19 d20 - disk)
  (:init (on d1 pega) (on d2 pega) (on d3 pega) (on d4 pega) (on d5 pega) (on d6 pega) (on d7 pega) (on d8 pega) (on d9 pega) (on d10 pega) (on d11 pega) (on d12 pega) (on d13 pega) (on d14 pega) (on d15 pega) (on d16 pega) (on d17 pega) (on d18 pega) (on d19 pega) (on d20 pega) 
         (larger d2 d1) 
         (larger d3 d1) (larger d3 d2) 
         (larger d4 d1) (larger d4 d2) (larger d4 d3) 
         (larger d5 d1) (larger d5 d2) (larger d5 d3) (larger d5 d4) 
         (larger d6 d1) (larger d6 d2) (larger d6 d3) (larger d6 d4) (larger d6 d5) 
         (larger d7 d1) (larger d7 d2) (larger d7 d3) (larger d7 d4) (larger d7 d5) (larger d7 d6) 
         (larger d8 d1) (larger d8 d2) (larger d8 d3) (larger d8 d4) (larger d8 d5) (larger d8 d6) (larger d8 d7) 
         (larger d9 d1) (larger d9 d2) (larger d9 d3) (larger d9 d4) (larger d9 d5) (larger d9 d6) (larger d9 d7) (larger d9 d8) 
         (larger d10 d1) (larger d10 d2) (larger d10 d3) (larger d10 d4) (larger d10 d5) (larger d10 d6) (larger d10 d7) (larger d10 d8) (larger d10 d9) 
         (larger d11 d1) (larger d11 d2) (larger d11 d3) (larger d11 d4) (larger d11 d5) (larger d11 d6) (larger d11 d7) (larger d11 d8) (larger d11 d9) (larger d11 d10) 
         (larger d12 d1) (larger d12 d2) (larger d12 d3) (larger d12 d4) (larger d12 d5) (larger d12 d6) (larger d12 d7) (larger d12 d8) (larger d12 d9) (larger d12 d10) (larger d12 d11) 
         (larger d13 d1) (larger d13 d2) (larger d13 d3) (larger d13 d4) (larger d13 d5) (larger d13 d6) (larger d13 d7) (larger d13 d8) (larger d13 d9) (larger d13 d10) (larger d13 d11) (larger d13 d12) 
         (larger d14 d1) (larger d14 d2) (larger d14 d3) (larger d14 d4) (larger d14 d5) (larger d14 d6) (larger d14 d7) (larger d14 d8) (larger d14 d9) (larger d14 d10) (larger d14 d11) (larger d14 d12) (larger d14 d13) 
         (larger d15 d1) (larger d15 d2) (larger d15 d3) (larger d15 d4) (larger d15 d5) (larger d15 d6) (larger d15 d7) (larger d15 d8) (larger d15 d9) (larger d15 d10) (larger d15 d11) (larger d15 d12) (larger d15 d13) (larger d15 d14) 
         (larger d16 d1) (larger d16 d2) (larger d16 d3) (larger d16 d4) (larger d16 d5) (larger d16 d6) (larger d16 d7) (larger d16 d8) (larger d16 d9) (larger d16 d10) (larger d16 d11) (larger d16 d12) (larger d16 d13) (larger d16 d14) (larger d16 d15) 
         (larger d17 d1) (larger d17 d2) (larger d17 d3) (larger d17 d4) (larger d17 d5) (larger d17 d6) (larger d17 d7) (larger d17 d8) (larger d17 d9) (larger d17 d10) (larger d17 d11) (larger d17 d12) (larger d17 d13) (larger d17 d14) (larger d17 d15) (larger d17 d16) 
         (larger d18 d1) (larger d18 d2) (larger d18 d3) (larger d18 d4) (larger d18 d5) (larger d18 d6) (larger d18 d7) (larger d18 d8) (larger d18 d9) (larger d18 d10) (larger d18 d11) (larger d18 d12) (larger d18 d13) (larger d18 d14) (larger d18 d15) (larger d18 d16) (larger d18 d17) 
         (larger d19 d1) (larger d19 d2) (larger d19 d3) (larger d19 d4) (larger d19 d5) (larger d19 d6) (larger d19 d7) (larger d19 d8) (larger d19 d9) (larger d19 d10) (larger d19 d11) (larger d19 d12) (larger d19 d13) (larger d19 d14) (larger d19 d15) (larger d19 d16) (larger d19 d17) (larger d19 d18) 
         (larger d20 d1) (larger d20 d2) (larger d20 d3) (larger d20 d4) (larger d20 d5) (larger d20 d6) (larger d20 d7) (larger d20 d8) (larger d20 d9) (larger d20 d10) (larger d20 d11) (larger d20 d12) (larger d20 d13) (larger d20 d14) (larger d20 d15) (larger d20 d16) (larger d20 d17) (larger d20 d18) (larger d20 d19) 
         )
  (:goal (and (on d1 pegc) (on d2 pegc) (on d3 pegc) (on d4 pegc) (on d5 pegc) (on d6 pegc) (on d7 pegc) (on d8 pegc) (on d9 pegc) (on d10 pegc) (on d11 pegc) (on d12 pegc) (on d13 pegc) (on d14 pegc) (on d15 pegc) (on d16 pegc) (on d17 pegc) (on d18 pegc) (on d19 pegc) (on d20 pegc))))

